@@ -23,34 +23,50 @@ import random
 from gensim import utils
 
 
+logger = logging.getLogger('dictionary')
+logger.setLevel(logging.INFO)
+
+
 class Dictionary(utils.SaveLoad):
     """
     Dictionary encapsulates mappings between normalized words and their integer ids.
     
     The main function is `doc2bow`, which converts a collection of words to its 
-    bag-of-words representation, optionally also updating the dictionary mapping with new 
-    words and their ids.
+    bag-of-words representation, optionally also updating the dictionary mapping 
+    with newly encountered words and their ids.
     """
-    def __init__(self):
-        self.id2token = {} # tokenId -> token
+    def __init__(self, documents = None):
         self.token2id = {} # token -> tokenId
         self.docFreq = {} # tokenId -> in how many documents this token appeared
-        self.numDocs = 0
+        self.numDocs = 0 # number of documents processed
+        self.numPos = 0 # total number of corpus positions
+        
+        if documents:
+            self.addDocuments(documents)
+    
+    
+    # expensive, only here for historical reasons; maybe deprecate?
+    id2token = property(lambda self: dict((id, token) for token, id in self.token2id.iteritems())) 
+    id2word = id2token
     
 
     def __len__(self):
         """
         Return the number of token->id mappings in the dictionary.
         """
-        assert len(self.token2id) == len(self.id2token)
         return len(self.token2id)
 
 
     def __str__(self):
         return ("Dictionary(%i unique tokens)" % len(self))
 
+
     @staticmethod
     def fromDocuments(documents):
+        return Dictionary(documents = documents)
+
+
+    def addDocuments(self, documents):
         """
         Build dictionary from a collection of documents. Each document is a list 
         of tokens (ie. **tokenized and normalized** utf-8 encoded strings).
@@ -61,19 +77,13 @@ class Dictionary(utils.SaveLoad):
         >>> print Dictionary.fromDocuments(["máma mele maso".split(), "ema má máma".split()])
         Dictionary(5 unique tokens)
         """
-        result = Dictionary()
-        for document in documents:
-            _ = result.doc2bow(document, allowUpdate = True) # ignore the result, here we only care about updating token ids
-        return result
-
-
-    def addToken(self, token, tokenId):
-        if tokenId in self.id2token:
-            logging.debug("overwriting old token %s (id %i); is this intended?" %
-                          (token, tokenId))
-        self.id2token[tokenId] = token
-        self.token2id[token] = tokenId
-    
+        for docno, document in enumerate(documents):
+            if docno % 10000 == 0:
+                logging.info("adding document #%i to %s" % (docno, self))
+            _ = self.doc2bow(document, allowUpdate = True) # ignore the result, here we only care about updating token ids
+        logger.info("built %s from %i documents (total %i corpus positions)" % 
+                     (self, self.numDocs, self.numPos))
+        
     
     def doc2bow(self, document, allowUpdate = False):
         """
@@ -88,7 +98,7 @@ class Dictionary(utils.SaveLoad):
         If `allowUpdate` is **not** set, this function is `const`, ie. read-only.
         """
         result = {}
-        
+        document = sorted(document)
         # construct (word, frequency) mapping. in python3 this is done simply 
         # using Counter(), but here i use itertools.groupby() for the job
         for wordNorm, group in itertools.groupby(sorted(document)):
@@ -99,14 +109,15 @@ class Dictionary(utils.SaveLoad):
                 # first time we see this token (~normalized form)
                 if not allowUpdate: # if we aren't allowed to create new tokens, continue with the next unique token
                     continue
-                tokenId = len(self.token2id) # new id = number of ids made so far; NOTE this assumes there are no gaps in the id sequence!
-                self.addToken(wordNorm, tokenId)
+                tokenId = len(self.token2id)
+                self.token2id[wordNorm] = tokenId # new id = number of ids made so far; NOTE this assumes there are no gaps in the id sequence!
             
             # update how many times a token appeared in the document
-            result[tokenId] = result.get(tokenId, 0) + frequency
-            
+            result[tokenId] = frequency
+        
         if allowUpdate:
             self.numDocs += 1
+            self.numPos += len(document)
             # increase document count for each unique token that appeared in the document
             for tokenId in result.iterkeys():
                 self.docFreq[tokenId] = self.docFreq.get(tokenId, 0) + 1
@@ -114,49 +125,52 @@ class Dictionary(utils.SaveLoad):
         return sorted(result.iteritems()) # return tokenIds, in ascending id order
 
 
-    def filterExtremes(self, noBelow = 5, noAbove = 0.5):
+    def filterExtremes(self, noBelow = 5, noAbove = 0.5, keepN = None):
         """
         Filter out tokens that appear in
         
         1. less than `noBelow` documents (absolute number) or 
         2. more than `noAbove` documents (fraction of total corpus size, *not* 
            absolute number).
+        3. after (1) and (2), keep only the first `keepN' most frequent tokens (or
+           all if `None`).
         
         After the pruning, shrink resulting gaps in word ids. 
         
-        **Note**: The same word may have a different word id before and after
-        the call to this function!
+        **Note**: Due to the gap shrinking, the same word may have a different 
+        word id before and after the call to this function!
         """
         noAboveAbs = int(noAbove * self.numDocs) # convert fractional threshold to absolute threshold
         
-        # determine which tokens to drop
-        badIds = [tokenId for tokenId, docFreq in self.docFreq.iteritems() if docFreq < noBelow or docFreq > noAboveAbs]
-        logging.info("removing %i tokens which were in less than %i or more than %i (=%.1f%%) documents" %
-                     (len(badIds), noBelow, noAboveAbs, 100.0 * noAbove))
-        
-        # print some sanity check debug info
-        if len(badIds) >= 10:
-            someIds = random.sample(badIds, 10) # choose 10 random ids that will be removed
-            someTokenFreqs = [(self.id2token[tokenId], self.docFreq[tokenId]) for tokenId in someIds]
-            logging.info("document frequencies of some of the removed tokens: [%s]" % 
-                         ', '.join("%s:%i" % i for i in someTokenFreqs))
+        # determine which tokens to keep
+        goodIds = (v for v in self.token2id.itervalues() if noBelow <= self.docFreq[v] <= noAboveAbs)
+        goodIds = sorted(goodIds, key = self.docFreq.get, reverse = True)
+        if keepN is not None:
+            goodIds = goodIds[:keepN]
+        logger.info("keeping %i tokens which were in more than %i and less than %i (=%.1f%%) documents" %
+                     (len(goodIds), noBelow, noAboveAbs, 100.0 * noAbove))
         
         # do the actual filtering, then rebuild dictionary to remove gaps in ids
-        self.filterTokens(badIds)
+        self.filterTokens(goodIds = goodIds)
         self.rebuildDictionary()
-        logging.info("resulting dictionary: %s" % self)
+        logger.info("resulting dictionary: %s" % self)
 
     
-    def filterTokens(self, badIds):
+    def filterTokens(self, badIds = None, goodIds = None):
         """
-        Remove the selected tokens from all dictionary mappings.
+        Remove the selected `badIds` tokens from all dictionary mappings, or, keep
+        selected `goodIds` in the mapping and remove the rest.
         
-        `badIds` is a collection of word ids to be removed.
+        `badIds` and `goodIds` are collections of word ids to be removed.
         """
-        badIds = set(badIds)
-        self.id2token = dict((tokenId, token) for tokenId, token in self.id2token.iteritems() if tokenId not in badIds)
-        self.token2id = dict((token, tokenId) for token, tokenId in self.token2id.iteritems() if tokenId not in badIds)
-        self.docFreq = dict((tokenId, freq) for tokenId, freq in self.docFreq.iteritems() if tokenId not in badIds)
+        if badIds is not None:
+            badIds = set(badIds)
+            self.token2id = dict((token, tokenId) for token, tokenId in self.token2id.iteritems() if tokenId not in badIds)
+            self.docFreq = dict((tokenId, freq) for tokenId, freq in self.docFreq.iteritems() if tokenId not in badIds)
+        if goodIds is not None:
+            goodIds = set(goodIds)
+            self.token2id = dict((token, tokenId) for token, tokenId in self.token2id.iteritems() if tokenId in goodIds)
+            self.docFreq = dict((tokenId, freq) for tokenId, freq in self.docFreq.iteritems() if tokenId in goodIds)
 
     
     def rebuildDictionary(self):
@@ -167,13 +181,12 @@ class Dictionary(utils.SaveLoad):
         been removed via :func:`filterTokens` and there are gaps in the id series.
         Calling this method will remove the gaps.
         """
-        logging.debug("rebuilding dictionary, shrinking gaps")
+        logger.debug("rebuilding dictionary, shrinking gaps")
         
         # build mapping from old id -> new id
-        idmap = dict(itertools.izip(self.id2token.iterkeys(), xrange(len(self.id2token))))
+        idmap = dict(itertools.izip(self.token2id.itervalues(), xrange(len(self.token2id))))
         
-        # reassign all mappings to new ids
-        self.id2token = dict((idmap[tokenId], token) for tokenId, token in self.id2token.iteritems())
+        # reassign mappings to new ids
         self.token2id = dict((token, idmap[tokenId]) for token, tokenId in self.token2id.iteritems())
         self.docFreq = dict((idmap[tokenId], freq) for tokenId, freq in self.docFreq.iteritems())
 #endclass Dictionary
