@@ -16,12 +16,42 @@ import numpy
 import scipy.sparse
 
 
+logger = logging.getLogger("matutils")
+logger.setLevel(logging.INFO)
+
+
+
+def corpus2csc(m, corpus, dtype = numpy.float64):
+    """
+    Convert corpus into a sparse matrix, in scipy.sparse.csc_matrix format.
+    
+    The corpus must not be empty (at least one document).
+    """
+    logger.info("constructing sparse document matrix")
+    # construct the sparse matrix as lil_matrix first, convert to csc later
+    # lil_matrix can quickly update rows, so initialize it transposed (documents=rows)
+    mat = scipy.sparse.lil_matrix((1, 1), dtype = dtype)
+    mat.rows, mat.data = [], []
+    for i, doc in enumerate(corpus):
+        doc = sorted(doc)
+        mat.rows.append([fid for fid, _ in doc])
+        mat.data.append([val for _, val in doc])
+    docs = i + 1
+    mat._shape = (docs, m)
+    mat = mat.tocsr().transpose() # transpose back to documents=columns
+    assert isinstance(mat, scipy.sparse.csc_matrix)
+    return mat
+
+
 def pad(mat, padRow, padCol):
     """
     Add additional rows/columns to a numpy.matrix `mat`. The new rows/columns 
     will be initialized with zeros.
     """
-    assert padRow >= 0, padCol >= 0
+    if padRow < 0:
+        padRow = 0
+    if padCol < 0:
+        padCol = 0
     rows, cols = mat.shape
     return numpy.bmat([[mat, numpy.matrix(numpy.zeros((rows, padCol)))],
                       [numpy.matrix(numpy.zeros((padRow, cols + padCol)))]])
@@ -32,7 +62,7 @@ def sparse2full(doc, length):
     Convert document in sparse format (sequence of 2-tuples) into a full numpy
     array (of size `length`).
     """
-    result = numpy.zeros(length, dtype = numpy.float32) # fill with zeroes (default value)
+    result = numpy.zeros(length, dtype = numpy.float32, order = 'F') # fill with zeroes (default value)
     doc = dict(doc)
     result[doc.keys()] = doc.values() # overwrite some of the zeroes with explicit values
     return result
@@ -114,10 +144,10 @@ class MmWriter(object):
         
         if numNnz < 0:
             # we don't know the matrix shape/density yet, so only log a general line
-            logging.info("saving sparse matrix to %s" % self.fname)
+            logger.info("saving sparse matrix to %s" % self.fname)
             self.fout.write(' ' * 50 + '\n') # 48 digits must be enough for everybody
         else:
-            logging.info("saving sparse %sx%s matrix with %i non-zero entries to %s" %
+            logger.info("saving sparse %sx%s matrix with %i non-zero entries to %s" %
                          (numDocs, numTerms, numNnz, self.fname))
             self.fout.write('%s %s %s\n' % (numDocs, numTerms, numNnz))
         self.lastDocNo = -1
@@ -144,12 +174,12 @@ class MmWriter(object):
             if weight == 0:
                 # to ensure len(doc) does what is expected, there must not be any zero elements in the sparse document
                 raise ValueError("zero weights not allowed in sparse documents; check your document generator")
-            self.fout.write("%i %i %f\n" % (docNo + 1, termId + 1, weight)) # +1 because MM format starts counting from 1
+            self.fout.write("%i %i %s\n" % (docNo + 1, termId + 1, weight)) # +1 because MM format starts counting from 1
         self.lastDocNo = docNo
 
 
     @staticmethod
-    def writeCorpus(fname, corpus):
+    def writeCorpus(fname, corpus, progressCnt = 1000):
         """
         Save the vector space representation of an entire corpus to disk.
         
@@ -162,20 +192,19 @@ class MmWriter(object):
         mw.writeHeaders(-1, -1, -1) # will print 50 spaces followed by newline on the stats line
         
         # calculate necessary header info (nnz elements, num terms, num docs) while writing out vectors
-        numDocs = len(corpus)
         numTerms = numNnz = 0
         
         for docNo, bow in enumerate(corpus):
-            if docNo % 1000 == 0:
-                logging.info("PROGRESS: saving document %i/%i" % 
-                             (docNo, len(corpus)))
+            if docNo % progressCnt == 0:
+                logger.info("PROGRESS: saving document %i" % docNo)
             if len(bow) > 0:
                 numTerms = max(numTerms, 1 + max(wordId for wordId, val in bow))
                 numNnz += len(bow)
             mw.writeVector(docNo, bow)
+        numDocs = docNo + 1
             
         if numDocs * numTerms != 0:
-            logging.info("saved %ix%i matrix, density=%.3f%% (%i/%i)" % 
+            logger.info("saved %ix%i matrix, density=%.3f%% (%i/%i)" % 
                          (numDocs, numTerms,
                           100.0 * numNnz / (numDocs * numTerms),
                           numNnz,
@@ -211,30 +240,34 @@ class MmReader(object):
     as an object which supports iteration over the rows (~documents).
     
     Note that the file is read into memory one document at a time, not the whole 
-    matrix at once (unlike scipy.io.mmread). This allows for representing corpora 
+    matrix at once (unlike scipy.io.mmread). This allows us to process corpora 
     which are larger than the available RAM.
     """
-    def __init__(self, fname):
+    def __init__(self, input):
         """
         Initialize the matrix reader. 
         
-        The `fname` is a path to a file on local filesystem, which is expected to 
-        be in sparse (coordinate) Matrix Market format. Documents are assumed to 
-        be rows of the matrix (and document features are columns).
+        The `input` refers to a file on local filesystem, which is expected to 
+        be in the sparse (coordinate) Matrix Market format. Documents are assumed 
+        to be rows of the matrix (and document features are columns). 
+        
+        `input` is either a string (file path) or a file-like object that supports
+        `seek(0)` (e.g. gzip.GzipFile, bz2.BZ2File).
         """
-        logging.info("initializing corpus reader from %s" % fname)
-        self.fname = fname
-        fin = open(fname)
-        header = fin.next().strip()
+        logger.info("initializing corpus reader from %s" % input)
+        self.input = input
+        if isinstance(input, basestring):
+            input = open(input)
+        header = input.next().strip()
         if not header.lower().startswith('%%matrixmarket matrix coordinate real general'):
             raise ValueError("File %s not in Matrix Market format with coordinate real general; instead found: \n%s" % 
-                             (fname, header))
+                             (self.input, header))
         self.numDocs = self.numTerms = self.numElements = 0
-        for lineNo, line in enumerate(fin):
+        for lineNo, line in enumerate(input):
             if not line.startswith('%'):
                 self.numDocs, self.numTerms, self.numElements = map(int, line.split())
                 break
-        logging.info("accepted corpus with %i documents, %i terms, %i non-zero entries" %
+        logger.info("accepted corpus with %i documents, %i terms, %i non-zero entries" %
                      (self.numDocs, self.numTerms, self.numElements))
     
     def __len__(self):
@@ -254,8 +287,11 @@ class MmReader(object):
         yielded where appropriate, even if they are not explicitly stored in the 
         Matrix Market file.
         """
-        fin = open(self.fname)
-        
+        if isinstance(self.input, basestring):
+            fin = open(self.input)
+        else:
+            fin = self.input
+            fin.seek(0)
         # skip headers
         for line in fin:
             if line.startswith('%'):
